@@ -62,19 +62,14 @@ class ApplianceImageCreator(ImageCreator):
         #self.getsource = False
         #self.listpkg = False
 
-        self._dep_checks.extend([
-                            ('/usr/sbin/sync', '/sbin/sync', '/bin/sync', '/usr/bin/sync'),
-                            ('/sbin/kpartx',),
-                            ('/sbin/parted', '/usr/sbin/parted'),
-                            ('/sbin/extlinux', '/usr/sbin/extlinux', '/usr/bin/extlinux')
-                           ])
+        self._dep_checks.extend(["sync", "kpartx", "parted", "extlinux"])
 
         if self.__disk_format == "vmdk":
-            self._dep_checks.append(('/usr/bin/qemu-img',))
+            self._dep_checks.append("qemu-img")
         if self.__disk_format == "vdi":
-            self._dep_checks.append(('/usr/bin/VBoxManage',))
+            self._dep_checks.append("VBoxManage")
 
-    def configure(self):
+    def configure(self, repodata = None):
         def chroot():
 
             os.chroot(self._instroot)
@@ -83,7 +78,7 @@ class ApplianceImageCreator(ImageCreator):
         if self.__disk_format in ("raw", "vmdk", "vdi"):
             if os.path.exists(self._instroot + "/usr/bin/Xorg"):
                 subprocess.call(["/bin/chmod", "u+s", "/usr/bin/Xorg"], preexec_fn = chroot)
-        ImageCreator.configure(self)
+        ImageCreator.configure(self, repodata)
         
     def _get_fstab(self):
         s = ""
@@ -282,16 +277,12 @@ class ApplianceImageCreator(ImageCreator):
 
         # Set MBR
         mbrsize = os.stat("%s/usr/share/syslinux/mbr.bin" % self._instroot)[stat.ST_SIZE]
-        rc = subprocess.call(["/bin/dd", "if=%s/usr/share/syslinux/mbr.bin" % self._instroot, "of=" + loopdev])
+        rc = subprocess.call([ddcmd, "if=%s/usr/share/syslinux/mbr.bin" % self._instroot, "of=" + loopdev])
         if rc != 0:
             raise MountError("Unable to set MBR to %s" % loopdev)
 
         # Set Bootable flag
-        parted="/usr/sbin/parted"
-        if not os.path.exists(parted):
-            parted="/sbin/parted"
-            if not os.path.exists(parted):
-                raise CreatorError("Missed parted, please install it.")
+        parted = find_binary_path("parted")
         dev_null = os.open("/dev/null", os.O_WRONLY)
         rc = subprocess.call([parted, "-s", loopdev, "set", "%d" % (bootdevnum + 1), "boot", "on"],
                              stdout = dev_null, stderr = dev_null)
@@ -306,13 +297,7 @@ class ApplianceImageCreator(ImageCreator):
         # Ensure all data is flushed to disk before doing syslinux install
         subprocess.call(["sync"])
 
-        fullpathsyslinux = "/sbin/extlinux"
-        if not os.path.isfile(fullpathsyslinux):
-            fullpathsyslinux = "/usr/sbin/extlinux"
-            if not os.path.isfile(fullpathsyslinux):
-                fullpathsyslinux = "/usr/bin/extlinux"
-                if not os.path.isfile(fullpathsyslinux):
-                    raise CreatorError("Missed extlinux, please install it.")
+        fullpathsyslinux = find_binary_path("extlinux")
         rc = subprocess.call([fullpathsyslinux, "-i", "%s/boot/extlinux" % self._instroot])
         if rc != 0:
             raise MountError("Unable to install syslinux bootloader to %sp)d" % (loopdev, (bootdevnum + 1)))
@@ -328,87 +313,6 @@ class ApplianceImageCreator(ImageCreator):
     def _resparse(self, size = None):
         return self.__instloop.resparse(size)
 
-    def package(self, destdir,package = "none",include = None):
-        """Prepares the created image for final delivery.
-           Stage
-           add includes
-           package
-        """
-        self._stage_final_image()
-
-        if self._recording_pkgs:
-            self._save_recording_pkgs(destdir)
-
-        #add stuff
-        if include and os.path.isdir(include):
-            logging.debug("adding everything in %s to %s" % (include,self._outdir))
-            files = glob.glob('%s/*' % include)
-            for file in files:
-                if os.path.isdir(file):
-                    logging.debug("adding dir %s to %s" % (file,os.path.join(self._outdir,os.path.basename(file))))
-                    shutil.copytree(file, os.path.join(self._outdir,os.path.basename(file)),symlinks=False)
-                else:
-                    logging.debug("adding %s to %s" % (file,self._outdir))
-                    shutil.copy(file, self._outdir)
-        elif include:
-            logging.debug("adding %s to %s" % (include,self._outdir))
-            shutil.copy(include, self._outdir)
-
-        #package
-        (pkg, comp) = os.path.splitext(package)
-        if comp:
-            comp=comp.lstrip(".")
-
-        if pkg == "zip":
-            dst = "%s/%s.zip" % (destdir, self.name)
-            files = glob.glob('%s/*' % self._outdir)
-            if comp == "64":
-                logging.debug("creating %s with ZIP64 extensions" %  (dst))
-                z = zipfile.ZipFile(dst, "w", compression=8, allowZip64="True")
-            else:
-                logging.debug("creating %s" %  (dst))
-                z = zipfile.ZipFile(dst, "w", compression=8, allowZip64="False")
-            for file in files:
-                if file != dst:
-                    if os.path.isdir(file):
-                        #because zip sucks we cannot just add a dir
-                         for root, dirs, dirfiles in os.walk(file):
-                             for dirfile in dirfiles:
-                                 arcfile=self.name+"/"+root[len(os.path.commonprefix((os.path.dirname(file), root)))+1:]+"/"+dirfile
-                                 filepath=os.path.join(root,dirfile)
-                                 logging.debug("adding %s to %s" % (arcfile,dst))
-                                 z.write(filepath,arcfile, compress_type=None)
-                    else:
-                        logging.debug("adding %s to %s" % (os.path.join(self.name,os.path.basename(file)),dst))
-                        z.write(file, arcname=os.path.join(self.name,os.path.basename(file)), compress_type=None)
-            z.close()
-
-        elif pkg == "tar":
-            if comp:
-                dst = "%s/%s.tar.%s" % (destdir, self.name, comp)
-            else:
-                dst = "%s/%s.tar" % (destdir, self.name)
-            files = glob.glob('%s/*' % self._outdir)
-            logging.debug("creating %s" %  (dst))
-            tar = tarfile.open(dst, "w|"+comp)
-            for file in files:
-                logging.debug("adding %s to %s" % (file,dst))
-                tar.add(file, arcname=os.path.join(self.name,os.path.basename(file)))
-            tar.close()
-
-        else:
-            dst = os.path.join(destdir, "%s-%s" % (self.name, self.__disk_format))
-            logging.debug("creating destination dir: " + dst)
-            makedirs(dst)
-            # Ensure all data is flushed to _outdir
-            subprocess.call(["sync"])
-            for f in os.listdir(self._outdir):
-                logging.debug("moving %s to %s" % (os.path.join(self._outdir, f), os.path.join(dst, f)))
-                shutil.move(os.path.join(self._outdir, f),os.path.join(dst, f))
-                self.outimage.append(os.path.join(dst, f))
-                if f[-4:] not in (".vmx", ".xml"):
-                    self.do_genchecksum(os.path.join(dst, f))
-
     def _stage_final_image(self):
         """Stage the final system image in _outdir.
            Convert disks
@@ -424,7 +328,8 @@ class ApplianceImageCreator(ImageCreator):
             logging.debug("moving disks to stage location")
             for name in self.__disks.keys():
                 src = "%s/%s-%s.raw" % (self.__imgdir, self.name,name)
-                dst = "%s/%s-%s.%s" % (self._outdir, self.name,name, self.__disk_format)
+                self._img_name = "%s-%s.%s" % (self.name, name, self.__disk_format)
+                dst = "%s/%s" % (self._outdir, self._img_name)
                 logging.debug("moving %s to %s" % (src,dst))
                 shutil.move(src,dst)
         #write meta data in stage dir
@@ -434,14 +339,17 @@ class ApplianceImageCreator(ImageCreator):
     def _convert_image(self):
         #convert disk format
         for name in self.__disks.keys():
-            dst = "%s/%s-%s.%s" % (self._outdir, self.name,name, self.__disk_format)
+            self._img_name = "%s-%s.%s" % (self.name, name, self.__disk_format)
+            dst = "%s/%s" % (self._outdir, self._img_name)
             logging.debug("converting %s image to %s" % (self.__disks[name].lofile, dst))
             if self.__disk_format != "vdi":
-                rc = subprocess.call(["/usr/bin/qemu-img", "convert",
+                qemu_img = find_binary_path("qemu-img")
+                rc = subprocess.call([qemu_img, "convert",
                                       "-f", "raw", self.__disks[name].lofile,
                                       "-O", self.__disk_format,  dst])
             else:
-                rc = subprocess.call(["/usr/bin/VBoxManage", "convertfromraw",
+                vboxmanage = find_binary_path("VBoxManage")
+                rc = subprocess.call([vboxmanage, "convertfromraw",
                                       self.__disks[name].lofile, dst,
                                       "--format", "VDI"])
             if rc == 0:
@@ -504,6 +412,9 @@ ide0:0.redo = ""
         vmdkcfg.close()
 
     def _write_image_xml(self):
+        imgarch = "i686"
+        if self.target_arch and self.target_arch.startswith("arm"):
+            imgarch = "arm"
         xml = "<image>\n"
 
         name_attributes = ""
@@ -517,7 +428,7 @@ ide0:0.redo = ""
         # baremetal vs xen
         xml += "    <boot type='hvm'>\n"
         xml += "      <guest>\n"
-        xml += "        <arch>%s</arch>\n" % os.uname()[4]
+        xml += "        <arch>%s</arch>\n" % imgarch
         xml += "      </guest>\n"
         xml += "      <os>\n"
         xml += "        <loader dev='hd'/>\n"
