@@ -143,7 +143,7 @@ class NandImageCreator(ApplianceImageCreator):
         if self.__kernel_url or \
            self.__initrd_url or \
            self.__kernel_rpm_url:
-            self._dep_checks.append(('/usr/bin/curl', '/bin/curl'))
+            self._dep_checks.append("curl")
 
     #
     # Private helper functions
@@ -246,6 +246,7 @@ class NandImageCreator(ApplianceImageCreator):
 
         # create bootimg file
         imgpath = os.path.join(self._outdir, '%s-boot.bin' % self.name)
+        
         try:
             imgfile = open(imgpath, 'wb')
         except IOError, e:
@@ -281,6 +282,9 @@ class NandImageCreator(ApplianceImageCreator):
             raise CreatorError("IO error: %s" % e)
         finally:
             imgfile.close()
+            os.remove(bootstub)
+            os.remove(vmlinuz)
+            os.remove(initrd)
 
     #
     # Actual implementation
@@ -418,10 +422,10 @@ class NandImageCreator(ApplianceImageCreator):
             ayum.close()
             os.unlink(yum_conf)
 
-    def configure(self):
+    def configure(self, repodata = None):
         if not self.__bootimg_only:
             # call root class
-            ImageCreator.configure(self)
+            ImageCreator.configure(self, repodata)
 
             # copy initrd to instroot
             if (self.__initrd_url or self.__initrd_path) and self.__override_initrd:
@@ -434,25 +438,11 @@ class NandImageCreator(ApplianceImageCreator):
                     shutil.copyfile(initrd_path, initrd_path_in_root)
                 except IOError:
                     raise CreatorError("Unable to copy initrd files to instroot")
+        else:
+            # if bootimg only, only call this
+            self._create_bootconfig()
 
-            return
-
-        # if bootimg only, only call this
-        self._create_bootconfig()
-
-    def package(self, destdir = "."):
-        """Prepares the created image for final delivery.
-        """
-
-        if not self.__bootimg_only:
-            # parent _stage_final_image move image file from _imgdir to _outdir
-            self._stage_final_image()
-
-            if self._recording_pkgs:
-                self._save_recording_pkgs(destdir)
-
-        """ Start to create boot image in _outdir
-        """
+        """ Start to create boot image in _outdir """
 
         # get the actual path of boot files
         bootstub = self._outdir + "/bootstub"
@@ -460,16 +450,8 @@ class NandImageCreator(ApplianceImageCreator):
         initrds  = glob.glob(self._outdir + "/initrd*")
 
         # cmdline to be written to image
-        #cmdline = "ro boot=usb single pci=noearly console=tty1 console=ttyMS0 earlyprintk=mrst loglevel=8 notsc "
+        # cmdline = "ro boot=usb single pci=noearly console=tty1 console=ttyMS0 earlyprintk=mrst loglevel=8 notsc "
         # read boot cmdline from ks file
         cmdline = self.ks.handler.bootloader.appendLine
 
         self.__bootable_nandimg(cmdline, bootstub, vmlinuzs[0], initrds[0])
-
-        for f in glob.glob("%s/%s-*" % (self._outdir, self.name)):
-            shutil.move(f, os.path.join(destdir, os.path.basename(f)))
-            self.outimage.append(os.path.join(destdir, os.path.basename(f)))
-            if f[-4:] in (".bin"):
-                self.do_genchecksum(os.path.join(destdir, os.path.basename(f)))
-
-        print "SUCCESS."
