@@ -42,7 +42,6 @@ from mic.imgcreate.misc import *
 
 def get_repo_metadata(repo, cachedir, reponame, proxies = {}, arch = None):
     makedirs(cachedir + "/" + reponame)
-    print "Geting repomd.xml from repo %s..." % reponame
     url = str(repo + "/repodata/repomd.xml")
     filename = str("%s/%s/repomd.xml" % (cachedir, reponame))
     repomd = myurlgrab(url, filename, proxies)
@@ -58,7 +57,6 @@ def get_repo_metadata(repo, cachedir, reponame, proxies = {}, arch = None):
         primaryxml = str(cachedir + "/%s/%s" % (reponame, os.path.basename(m.group(1))))
         primarydb = cachedir + "/%s/%s" % (reponame, ".tmp.primary.sqlite")
         primaryxmlurl = str(repo + "/" + m.group(1))
-        print "Geting primary.xml from repo %s..." % reponame
         primaryxml = myurlgrab(primaryxmlurl, primaryxml, proxies)
         if primaryxml.rfind(".gz") != -1:
             subprocess.call([gunzip, "-f", primaryxml])
@@ -72,7 +70,6 @@ def get_repo_metadata(repo, cachedir, reponame, proxies = {}, arch = None):
            primarydb = primarydb + ".gz"
         elif primarydburl.rfind(".bz2") != -1:
            primarydb = primarydb + ".bz2"
-        print "Geting primary.sqlite from repo %s..." % reponame
         primarydb = myurlgrab(primarydburl, primarydb, proxies)
         if primarydb.rfind(".gz") != -1:
             subprocess.call([gunzip, "-f", primarydb])
@@ -176,7 +173,7 @@ def gen_sqlitedb_from_xml(primaryxml, sqlitedb, target_arch = None):
             if not target_arch and arch.startswith("arm"):
                 continue
             if target_arch != None and arch != target_arch and arch != "noarch":
-		continue
+                continue
             if arch == "src":
                 continue
             sql = "INSERT INTO packages VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)"
@@ -282,28 +279,42 @@ def get_my_all_deps_in_order(pkg, con):
 
     return childpkgs
 
-def is_ubuntu():
+def find_key_in_file(key, file):
     found = False
-    for release in glob.glob("/etc/*-release"):
-        fd = open(release, "r")
-        content = fd.read()
-        for line in content.split("\n"):
-            if line.find("Ubuntu") != -1:
-                found = True
-                break
-        fd.close()
-        if found:
+    if not os.path.exists(file):
+        return found
+    fd = open(file, "r")
+    content = fd.read()
+    for line in content.split("\n"):
+        if line.find(key) != -1:
+            found = True
             break
-
+    fd.close()
     return found
 
+def get_distribution():
+    import platform
+    try:
+        # Python 2.6
+        dist = platform.linux_distribution()[0]
+    except AttributeError:
+        dist = platform.dist()[0]
+
+    return dist
+
+def rpm_has_force_debian_option():
+    rpm = find_binary_path("rpm")
+    dev_null = os.open("/dev/null", os.O_WRONLY)
+    argv = [rpm, "--force-debian"]
+    ret = subprocess.call(argv, stdout = dev_null, stderr = dev_null)
+    os.close(dev_null)
+    if ret != 0:
+        return False
+    return True
+
 def install_rpm(pkgpath, pkgname, rootpath, force = False):
-    rpm = "/bin/rpm"
-    if not os.path.exists(rpm):
-        rpm = "/usr/bin/rpm"
-        if not os.path.exists(rpm):
-            raise CreatorError("Failed to run 'rpm'.")
-    
+    rpm = find_binary_path("rpm")
+
     rootpath = os.path.abspath(os.path.expanduser(rootpath))
     print "Installing %s..." % pkgname
     dev_null = os.open("/dev/null", os.O_WRONLY)
@@ -311,8 +322,10 @@ def install_rpm(pkgpath, pkgname, rootpath, force = False):
         argv = [rpm, "-i", "--ignorearch", "--ignoreos", "--nodigest", "--nosignature", "--root=" + rootpath, pkgpath]
         if force:
             argv.extend(["--force", "--nodeps"])
-        if is_ubuntu():
-            argv.append("--force-debian")
+        dist = get_distribution()
+        if dist == "Ubuntu" or dist == "debian":
+            if rpm_has_force_debian_option():
+                argv.append("--force-debian")
         subprocess.call(argv, stdout = dev_null, stderr = dev_null)
     finally:
         os.close(dev_null)
@@ -367,7 +380,7 @@ def build_bootstrap(repourl, cachedir, reponame, bootstrapdir, proxies = {}, arc
     for pkg in pkglist:
         download_and_install(pkg, True)
 
-    pkglist = ["glibc", "passwd", "pam-modules-cracklib", "meego-release", "moblin-release", "fastinit", "nss", "genisoimage", "bzip2", "gzip", "cpio", "perl", "syslinux-extlinux", "mic2"]
+    pkglist = ["glibc", "passwd", "pam-modules-cracklib", "meego-release", "moblin-release", "fastinit", "nss", "genisoimage", "bzip2", "gzip", "cpio", "perl", "syslinux-extlinux", "btrfs-progs", "make", "file", "psmisc", "mic2"]
     pkglist.extend(["isomd5sum", "wget"])
     for pkg in pkglist:
         for node in get_my_all_deps_in_order(pkg, con):
@@ -403,7 +416,7 @@ def is_mainrepo(cachedir, reponame):
     f = open(repomd, "r")
     content = f.read()
     f.close()
-    m = re.match(".*href=\"(.*?comps.xml.*?)\"", content, re.M | re.S)
+    m = re.match(".*href=\"(.*?comps.*?\.xml.*?)\"", content, re.M | re.S)
     if not m:
         return False
     sqlitedb = "%s/%s/.tmp.primary.sqlite" % (cachedir, reponame)
@@ -418,11 +431,9 @@ def is_mainrepo(cachedir, reponame):
 
 def check_depends(imgfmt):
     if imgfmt == "vmdk":
-        if not os.path.exists('/usr/bin/qemu-img'):
-            raise CreatorError("Needed file: /usr/bin/qemu-img not found, please check your installation.")
+        find_binary_path("qemu-img")
     if imgfmt == "vdi":
-        if not os.path.exists('/usr/bin/VBoxManage'):
-            raise CreatorError("Needed file: /usr/bin/VBoxManage not found, please check your installation..")
+        find_binary_path("VBoxManage")
 
 def save_imginfo(outimage, infofile):
     print "Saving image info..."
@@ -501,11 +512,13 @@ def convert_to(imgfile, imgfmt):
     outimage.append(dst)
     print "converting %s image to %s" % (imgfile, dst)
     if imgfmt == "vmdk":
-        rc = subprocess.call(["/usr/bin/qemu-img", "convert",
+        qemu_img = find_binary_path("qemu-img")
+        rc = subprocess.call([qemu_img, "convert",
                                       "-f", "raw", imgfile,
                                       "-O", imgfmt, dst])
     elif imgfmt == "vdi":
-        rc = subprocess.call(["/usr/bin/VBoxManage", "convertfromraw",
+        vboxmanage = find_binary_path("VBoxManage")
+        rc = subprocess.call([vboxmanage, "convertfromraw",
                                       imgfile, dst,
                                       "--format", "VDI"])
                 
@@ -556,3 +569,53 @@ def print_imginfo(outimage):
     print "Your new image can be found here:"
     for file in outimage:
         print os.path.abspath(file)
+
+def clean_files(pattern, dir):
+    for f in os.listdir(dir):
+        entry = os.path.join(dir, f)
+        if os.path.isdir(entry):
+            clean_files(pattern, entry)
+        elif re.match(pattern, entry):
+            os.unlink(entry)
+
+def copy_mic2(bootstrap, bin_path = "/usr/bin", python_lib_path = "/usr/lib/python2.6/site-packages"):
+        bootstrap_mic = bootstrap + "/usr/lib/python2.6/site-packages/mic"
+        shutil.rmtree(bootstrap_mic, ignore_errors = True)
+        subprocess.call([copycmd, "-af", python_lib_path + "/mic", os.path.dirname(bootstrap_mic)])
+        clean_files(".*\.py[co]$", bootstrap_mic)
+        for file in glob.glob(bootstrap + "/usr/bin/mic-*"):
+            os.unlink(file)
+        for file in glob.glob(bootstrap + "/usr/bin/moblin-*"):
+            os.unlink(file)
+        os.unlink(bootstrap + "/usr/bin/mic")
+        for file in glob.glob(bin_path + "/mic-*"):
+            shutil.copy(file, bootstrap + "/usr/bin/" + os.path.basename(file))
+        for file in glob.glob(bin_path + "/moblin-*"):
+            if os.path.islink(file):
+                linkto = os.readlink(file)
+                os.symlink(linkto, bootstrap + "/usr/bin/" + os.path.basename(file))
+        shutil.copy(bin_path + "/mic", bootstrap + "/usr/bin/mic")
+
+def check_mic(bootstrap):
+    import distutils.sysconfig
+    pythonlib = distutils.sysconfig.get_python_lib()
+    bin_path = "/usr/bin"
+    if not os.path.exists(pythonlib + "/mic/imgcreate/creator.py"):
+        dist = get_distribution()
+        if dist == 'Ubuntu':
+            dev_null = os.open("/dev/null", os.O_WRONLY)
+            proc = subprocess.Popen(["/usr/bin/pycentral", "pycentraldir", "mic2"], 
+                                    stdout = subprocess.PIPE, stderr = dev_null)
+            output = proc.communicate()[0]
+            os.close(dev_null)
+            if proc.returncode != 0:
+                raise CreatorError("Failed to run /usr/bin/pycentral")
+            pycentraldir = output.split()[0]
+            pythonlib = pycentraldir
+            bin_path = "/usr/local/bin"
+        elif dist == 'debian':
+            if os.path.exists("/usr/share/python-support/mic2"):
+                pythonlib = "/usr/share/python-support/mic2"
+            elif os.path.exists("/usr/share/pyshared"):
+                pythonlib = "/usr/share/pyshared"
+    copy_mic2(bootstrap, bin_path = bin_path, python_lib_path = pythonlib)
