@@ -38,6 +38,7 @@ from mic.imgcreate.fs import *
 from mic.imgcreate.misc import *
 from mic.imgcreate.yuminst import *
 from mic.imgcreate import kickstart
+from mic.imgcreate.srcdownload import SrcDownload
 
 FSLABEL_MAXLEN = 32
 """The maximum string length supported for LoopImageCreator.fslabel."""
@@ -75,7 +76,7 @@ class ImageCreator(object):
 
         self.distro_name = "MeeGo"
 
-        """Output image file name"""
+        """Output image file names"""
         self.outimage = []
 
         """A flag to generate checksum"""
@@ -91,18 +92,18 @@ class ImageCreator(object):
         self.__builddir = None
         self.__bindmounts = []
 
+        """ Contains the compression method that is used to compress
+        the disk image after creation, e.g., bz2.
+        This value is set with compression_method function. """
+        self.__img_compression_method = None
 
         # dependent commands to check
-        self._dep_checks = [('/bin/ls',),
-                            ('/bin/bash',),
-                            ('/bin/cp',),
-                            ('/bin/echo',),
-                            ('/sbin/modprobe',),
-                            ('/usr/bin/passwd',)
-                           ]
+        self._dep_checks = ["ls", "bash", "cp", "echo", "modprobe", "passwd"]
 
         self._recording_pkgs = None
 
+        self._include_src = None
+
         self._local_pkgs_path = None
 
         # available size in root fs, init to 0
@@ -111,17 +112,37 @@ class ImageCreator(object):
         # target arch for non-x86 image
         self.target_arch = None
 
+        """ Name of the disk image file that is created. """
+        self._img_name = None
+
+        """ Image format """
+        self.image_format = None
+
+        """ Save qemu emulator file name in order to clean up it finally """
+        self.qemu_emulator = None
+
+        """ No ks provided when called by convertor, so skip the dependency check """
+        if self.ks:
+            """ If we have btrfs partition we need to check that we have toosl for those """
+            for part in self.ks.handler.partition.partitions:
+                if part.fstype and part.fstype == "btrfs":
+                    self._dep_checks.append("mkfs.btrfs")
+                    break
+
     def set_target_arch(self, arch):
         if not rpmUtils.arch.arches.has_key(arch):
             return False
         self.target_arch = arch
         if self.target_arch.startswith("arm"):
             for dep in self._dep_checks:
-                if "/sbin/extlinux" in dep:
+                if dep == "extlinux":
                     self._dep_checks.remove(dep)
-        self._dep_checks.append(("/usr/bin/qemu-arm-static",))
+
+            if not os.path.exists("/usr/bin/qemu-arm") or not is_statically_linked("/usr/bin/qemu-arm"):
+                self._dep_checks.append("qemu-arm-static")
+
         return True
-        
+
 
     def __del__(self):
         self.cleanup()
@@ -219,6 +240,9 @@ class ImageCreator(object):
         """
         shutil.move(self._instroot, self._outdir + "/" + self.name)
 
+    def get_installed_packages(self):
+        return self._pkgs_content.keys()
+
     def _save_recording_pkgs(self, destdir):
         """Save the list or content of installed packages to file.
         """
@@ -234,6 +258,7 @@ class ImageCreator(object):
         content = '\n'.join(pkgs)
         f.write(content)
         f.close()
+        self.outimage.append(namefile);
 
         # if 'content', save more details
         if self._recording_pkgs == 'content':
@@ -258,6 +283,7 @@ class ImageCreator(object):
                 content += '\n'
                 f.write(content)
             f.close()
+            self.outimage.append(contfile)
 
     def _get_required_packages(self):
         """Return a list of required packages.
@@ -290,7 +316,7 @@ class ImageCreator(object):
             rpm_name = os.path.basename(rpm_path)
             package_name = rpmUtils.miscutils.splitFilename(rpm_name)[0]
             excluded_packages += [package_name]
-        return excluded_packages	
+        return excluded_packages
 
     def _get_local_packages(self):
         """Return a list of rpm path to be local installed.
@@ -589,9 +615,9 @@ class ImageCreator(object):
         for d in ("/dev/pts", "/etc", "/boot", "/var/log", "/var/cache/yum", "/sys", "/proc", "/usr/bin"):
             makedirs(self._instroot + d)
 
-        if self.target_arch:
-            setup_qemu_emulator(self._instroot, self.target_arch)
-        
+        if self.target_arch and self.target_arch.startswith("arm"):
+            self.qemu_emulator = setup_qemu_emulator(self._instroot, self.target_arch)
+
         self.get_cachedir(cachedir)
 
         # bind mount system directories into _instroot
@@ -624,6 +650,8 @@ class ImageCreator(object):
         """
         try:
             os.unlink(self._instroot + "/etc/mtab")
+            if self.qemu_emulator:
+                os.unlink(self._instroot + self.qemu_emulator)
         except OSError:
             pass
 
@@ -742,7 +770,13 @@ class ImageCreator(object):
 
         yum_conf = self._mktemp(prefix = "yum.conf-")
 
-        ayum = LiveCDYum(self._recording_pkgs, target_arch = self.target_arch)
+        keep_record = None
+        if self._include_src:
+            keep_record = 'include_src'
+        if self._recording_pkgs in ('name', 'contents'):
+            keep_record = self._recording_pkgs
+
+        ayum = LiveCDYum(keep_record, target_arch = self.target_arch)
         ayum.setup(yum_conf, self._instroot)
 
         for repo in kickstart.get_repos(self.ks, repo_urls):
@@ -779,8 +813,9 @@ class ImageCreator(object):
             except yum.Errors.YumBaseError, e:
                 raise CreatorError("Unable to install: %s" % (e,))
         finally:
-            if self._recording_pkgs:
+            if keep_record:
                 self._pkgs_content = ayum.getAllContent()
+
             ayum.closeRpmDB()
             ayum.close()
             os.unlink(yum_conf)
@@ -819,14 +854,24 @@ class ImageCreator(object):
             try:
                 try:
                     subprocess.call([s.interp, script],
-                                    preexec_fn = preexec, env = env)
+                                    preexec_fn = preexec, env = env, stdout = sys.stdout, stderr = sys.stderr)
                 except OSError, (err, msg):
                     raise CreatorError("Failed to execute %%post script "
                                        "with '%s' : %s" % (s.interp, msg))
             finally:
                 os.unlink(path)
 
-    def configure(self):
+    def __save_repo_keys(self, repodata):
+        if not repodata:
+            return None
+        gpgkeydir = "/etc/pki/rpm-gpg"
+        makedirs(self._instroot + gpgkeydir)
+        for repo in repodata:
+            if repo["repokey"]:
+                repokey = gpgkeydir + "/RPM-GPG-KEY-%s" %  repo["name"]
+                shutil.copy(repo["repokey"], self._instroot + repokey)
+
+    def configure(self, repodata = None):
         """Configure the system image according to the kickstart.
 
         This method applies the (e.g. keyboard or network) configuration
@@ -851,14 +896,13 @@ class ImageCreator(object):
             kickstart.NetworkConfig(self._instroot).apply(ksh.network)
             kickstart.RPMMacroConfig(self._instroot).apply(self.ks)
             kickstart.DesktopConfig(self._instroot).apply(ksh.desktop)
-            kickstart.MoblinRepoConfig(self._instroot).apply(ksh.repo)        
+            self.__save_repo_keys(repodata)
+            kickstart.MoblinRepoConfig(self._instroot).apply(ksh.repo, repodata)
         except:
-            if self.target_arch:
-                pass
-            else:
-                raise CreatorError("Failed to apply configuration to image")
+            print "Failed to apply configuration to image"
+            raise
 
-        if not self.target_arch:
+        if not self.target_arch or not self.target_arch.startswith("arm"):
             self._create_bootconfig()
 
         self.__run_post_scripts()
@@ -892,6 +936,7 @@ class ImageCreator(object):
                 fd = open(image_name + ".md5sum", "w")
                 fd.write(md5sum)
                 fd.close()
+                self.outimage.append(image_name+".md5sum")
 
     def package(self, destdir = "."):
         """Prepares the created image for final delivery.
@@ -907,9 +952,34 @@ class ImageCreator(object):
         """
         self._stage_final_image()
 
+        if self.__img_compression_method:
+            if not self._img_name:
+                raise CreatorError("Image name not set.")
+            rc = None
+            img_location = os.path.join(self._outdir,self._img_name)
+            if self.__img_compression_method == "bz2":
+                print "Compressing %s with bzip2. Please wait..." % img_location
+                rc = subprocess.call([bzip2, "-f", img_location])
+                if rc:
+                    raise CreatorError("Failed to compress image %s with %s." % (img_location, self.__img_compression_method))
+                for bootimg in glob.glob(os.path.dirname(img_location) + "/*-boot.bin"):
+                    print "Compressing %s with bzip2. Please wait..." % bootimg
+                    rc = subprocess.call([bzip2, "-f", bootimg])
+                    if rc:
+                        raise CreatorError("Failed to compress image %s with %s." % (bootimg, self.__img_compression_method))
+
         if self._recording_pkgs:
             self._save_recording_pkgs(destdir)
 
+        """ For image formats with two or multiple image files, it will be better to put them under a directory """
+        if self.image_format in ("raw", "vmdk", "vdi", "nand", "mrstnand"):
+            destdir = os.path.join(destdir, "%s-%s" % (self.name, self.image_format))
+            logging.debug("creating destination dir: %s" % destdir)
+            makedirs(destdir)
+
+        # Ensure all data is flushed to _outdir
+        subprocess.call([synccmd])
+
         for f in os.listdir(self._outdir):
             shutil.move(os.path.join(self._outdir, f),
                         os.path.join(destdir, f))
@@ -932,13 +1002,13 @@ class ImageCreator(object):
 
     def print_outimage_info(self):
         print "Your new image can be found here:"
+        self.outimage.sort()
         for file in self.outimage:
             print os.path.abspath(file)
 
     def check_depend_tools(self):
         for tool in self._dep_checks:
-            if tool and not any(map(os.path.exists, tool)):
-                raise CreatorError("Needed file: %s not found, please check your installation" % tool[0])
+            find_binary_path(tool)
 
     def package_output(self, image_format, destdir = ".", package="none"):
         if not package or package == "none":
@@ -956,17 +1026,23 @@ class ImageCreator(object):
                 dst = "%s/%s-%s.tar" % (destdir, self.name, image_format)
             print "creating %s" % dst
             tar = tarfile.open(dst, "w:" + comp)
+
             for file in self.outimage:
                 print "adding %s to %s" % (file, dst)
                 tar.add(file, arcname=os.path.join("%s-%s" % (self.name, image_format), os.path.basename(file)))
+                if os.path.isdir(file):
+                    shutil.rmtree(file, ignore_errors = True)
+                else:
+                    os.remove(file)
+
+
             tar.close()
-            self.outimage.append(dst)
+
+            '''All the file in outimage has been packaged into tar.* file'''
+            self.outimage = [dst]
 
     def release_output(self, config, destdir, name):
-        create_release(config, destdir, name)
-        self.outimage = []
-        for f in os.listdir(destdir):
-            self.outimage.append(os.path.join(destdir, f))
+        self.outimage = create_release(config, destdir, name, self.outimage)
 
     def save_kernel(self, destdir):
         if not os.path.exists(destdir):
@@ -976,6 +1052,17 @@ class ImageCreator(object):
             shutil.copy(kernel, kernelfilename)
             self.outimage.append(kernelfilename)
 
+    def compress_disk_image(self, compression_method):
+        """
+        With this you can set the method that is used to compress the disk
+        image after it is created.
+        """
+
+        if compression_method not in ('bz2'):
+            raise CreatorError("Given disk image compression method ('%s') is not valid." % (compression_method))
+
+        self.__img_compression_method = compression_method
+
 class LoopImageCreator(ImageCreator):
     """Installs a system into a loopback-mountable filesystem image.
 
@@ -1015,6 +1102,8 @@ class LoopImageCreator(ImageCreator):
         else:
             self.__image_size = 0
 
+        self._img_name = self.name + ".img"
+
     def _set_fstype(self, fstype):
         self.__fstype = fstype
 
@@ -1158,7 +1247,8 @@ class LoopImageCreator(ImageCreator):
 
     def _stage_final_image(self):
         self._resparse()
-        shutil.move(self._image, self._outdir + "/" + self.name + ".img")
+        shutil.move(self._image, self._outdir + "/" + self._img_name)
+
 
 
 class FsImageCreator(ImageCreator):
@@ -1176,7 +1266,7 @@ class FsImageCreator(ImageCreator):
         print "Moving %s to %s, please be patient to wait (it is slow if they are on different file systems/partitons/disks)" \
                % (self._instroot, destdir + "/" + self.name)
 
-        args = [ "/bin/cp", "-af", self._instroot + "/dev", os.path.dirname(self._instroot) ]
+        args = [ copycmd, "-af", self._instroot + "/dev", os.path.dirname(self._instroot) ]
         subprocess.call(args)
         shutil.rmtree(self._instroot + "/dev", ignore_errors = True)
 
@@ -1187,6 +1277,6 @@ class FsImageCreator(ImageCreator):
             if os.path.exists(destdir + "/" + self.name + exclude):
                 os.unlink(destdir + "/" + self.name + exclude)
 
-        args = [ "/bin/cp", "-af", os.path.dirname(self._instroot) + "/dev", destdir + "/" + self.name ]
+        args = [ copycmd, "-af", os.path.dirname(self._instroot) + "/dev", destdir + "/" + self.name ]
         subprocess.call(args)
         self.outimage.append(destdir + "/" + self.name)
