@@ -26,24 +26,79 @@ import random
 import string
 import logging
 import time
+import fcntl
+import struct
+import termios
 
 from mic.imgcreate.errors import *
 from urlgrabber.grabber import URLGrabber
 from urlgrabber.grabber import URLGrabError
 
-if os.path.exists('/bin/bunzip2'):
-    bunzip2 = '/bin/bunzip2'
-elif os.path.exists('/usr/bin/bunzip2'):
-    bunzip2 = '/usr/bin/bunzip2'
-else:
-    raise CreatorError('Failed to find bunzip2.')
+def terminal_width(fd=1):
+    """ Get the real terminal width """
+    try:
+        buf = 'abcdefgh'
+        buf = fcntl.ioctl(fd, termios.TIOCGWINSZ, buf)
+        return struct.unpack('hhhh', buf)[1]
+    except: # IOError
+        return 80
 
-if os.path.exists('/bin/gunzip'):
-    gunzip = '/bin/gunzip'
-elif os.path.exists('/usr/bin/gunzip'):
-    gunzip = '/usr/bin/gunzip'
-else:
-    raise CreatorError('Failed to find gunzip.')
+def truncate_url(url, width):
+    if len(url) > width:
+        return os.path.basename(url)[0:width]
+    return url
+
+class TextProgress(object):
+    def start(self, filename, url, *args, **kwargs):
+        self.url = url
+        self.termwidth = terminal_width()
+        sys.stdout.write("Retrieving %s " % truncate_url(self.url, self.termwidth - 13))
+        sys.stdout.flush()
+        self.indicators = ["-", "\\", "|", "/"]
+        self.counter = 0
+    def update(self, *args):
+        if sys.stdout.isatty():
+            sys.stdout.write("\rRetrieving %s %s" % (truncate_url(self.url, self.termwidth - 13), self.indicators[self.counter%4]))
+            sys.stdout.flush()
+            self.counter += 1
+        else:
+            pass
+    def end(self, *args):
+        if sys.stdout.isatty():
+            sys.stdout.write("\rRetrieving %s ...OK\n" % (self.url,))
+        else:
+            sys.stdout.write("...OK\n")
+        sys.stdout.flush()
+
+def find_binary_path(binary):
+    if os.environ.has_key("PATH"):
+        paths = os.environ["PATH"].split(":")
+    else:
+        paths = []
+        if os.environ.has_key("HOME"):
+            paths += [os.environ["HOME"] + "/bin"]
+        paths += ["/usr/local/sbin", "/usr/local/bin", "/usr/sbin", "/usr/bin", "/sbin", "/bin"]
+
+    for path in paths:
+        bin_path = "%s/%s" % (path, binary)
+        if os.path.exists(bin_path):
+            return bin_path
+    raise CreatorError("Command '%s' is not available." % binary)
+
+
+""" Get often-used bianry paths here """
+bunzip2 = find_binary_path('bunzip2')
+gunzip = find_binary_path('gunzip')
+bzip2 = find_binary_path('bzip2')
+copycmd = find_binary_path("cp")
+ddcmd = find_binary_path("dd")
+mountcmd = find_binary_path("mount")
+umountcmd = find_binary_path("umount")
+filecmd = find_binary_path("file")
+catcmd = find_binary_path("cat")
+synccmd = find_binary_path("sync")
+losetupcmd = find_binary_path("losetup")
+modprobecmd = find_binary_path("modprobe")
 
 def makedirs(dirname):
     """A version of os.makedirs() that doesn't throw an
@@ -56,11 +111,7 @@ def makedirs(dirname):
             raise
 
 def mksquashfs(in_img, out_img):
-    fullpathmksquashfs = "/sbin/mksquashfs"
-    if not os.path.isfile(fullpathmksquashfs):
-        fullpathmksquashfs = "/usr/bin/mksquashfs"
-        if not os.path.isfile(fullpathmksquashfs):
-            raise SquashfsError("Please install squashfs-tools")
+    fullpathmksquashfs = find_binary_path("mksquashfs")
     args = [fullpathmksquashfs, in_img, out_img]
 
     if not sys.stdout.isatty():
@@ -74,16 +125,15 @@ def mksquashfs(in_img, out_img):
 def resize2fs(fs, size):
     dev_null = os.open("/dev/null", os.O_WRONLY)
     try:
-        return subprocess.call(["/sbin/resize2fs", fs, "%sK" % (size / 1024,)],
+        resize2fs = find_binary_path("resize2fs")
+        return subprocess.call([resize2fs, fs, "%sK" % (size / 1024,)],
                                stdout = dev_null, stderr = dev_null)
     finally:
         os.close(dev_null)
 
 def my_fuser(file):
     ret = False
-    fuser = "/sbin/fuser"
-    if not os.path.exists(fuser):
-        fuser = "/bin/fuser"
+    fuser = find_binary_path("fuser")
     if not os.path.exists(file):
         return ret
     dev_null = os.open("/dev/null", os.O_WRONLY)
@@ -117,7 +167,7 @@ class BindChrootMount:
     def ismounted(self):
         ret = False
         dev_null = os.open("/dev/null", os.O_WRONLY)
-        args = [ "/bin/cat", "/proc/mounts" ]
+        args = [ catcmd, "/proc/mounts" ]
         proc_mounts = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=dev_null)
         outputs = proc_mounts.communicate()[0].strip().split("\n")
         for line in outputs:
@@ -136,12 +186,12 @@ class BindChrootMount:
             return
 
         makedirs(self.dest)
-        rc = subprocess.call(["/bin/mount", "--bind", self.src, self.dest])
+        rc = subprocess.call([mountcmd, "--bind", self.src, self.dest])
         if rc != 0:
             raise MountError("Bind-mounting '%s' to '%s' failed" %
                              (self.src, self.dest))
         if self.option:
-            rc = subprocess.call(["/bin/mount", "-o", "remount,%s" % self.option, self.dest])
+            rc = subprocess.call([mountcmd, "-o", "remount,%s" % self.option, self.dest])
             if rc != 0:
                 raise MountError("Bind-remounting '%s' failed" % self.dest)
         self.mounted = True
@@ -151,7 +201,7 @@ class BindChrootMount:
             return
 
         if self.ismounted():
-            subprocess.call(["/bin/umount", "-l", self.dest])
+            subprocess.call([umountcmd, "-l", self.dest])
         self.mounted = False
 
 class LoopbackMount:
@@ -168,7 +218,7 @@ class LoopbackMount:
 
     def lounsetup(self):
         if self.losetup:
-            rc = subprocess.call(["/sbin/losetup", "-d", self.loopdev])
+            rc = subprocess.call([losetupcmd, "-d", self.loopdev])
             self.losetup = False
             self.loopdev = None
 
@@ -176,7 +226,7 @@ class LoopbackMount:
         if self.losetup:
             return
 
-        losetupProc = subprocess.Popen(["/sbin/losetup", "-f"],
+        losetupProc = subprocess.Popen([losetupcmd, "-f"],
                                        stdout=subprocess.PIPE)
         losetupOutput = losetupProc.communicate()[0]
 
@@ -186,7 +236,7 @@ class LoopbackMount:
 
         self.loopdev = losetupOutput.split()[0]
 
-        rc = subprocess.call(["/sbin/losetup", self.loopdev, self.lofile])
+        rc = subprocess.call([losetupcmd, self.loopdev, self.lofile])
         if rc != 0:
             raise MountError("Failed to allocate loop device for '%s'" %
                              self.lofile)
@@ -297,7 +347,7 @@ class LoopbackDisk(Disk):
         if self.device is not None:
             return
 
-        losetupProc = subprocess.Popen(["/sbin/losetup", "-f"],
+        losetupProc = subprocess.Popen([losetupcmd, "-f"],
                                        stdout=subprocess.PIPE)
         losetupOutput = losetupProc.communicate()[0]
 
@@ -308,7 +358,7 @@ class LoopbackDisk(Disk):
         device = losetupOutput.split()[0]
 
         logging.debug("Losetup add %s mapping to %s"  % (device, self.lofile))
-        rc = subprocess.call(["/sbin/losetup", device, self.lofile])
+        rc = subprocess.call([losetupcmd, device, self.lofile])
         if rc != 0:
             raise MountError("Failed to allocate loop device for '%s'" %
                              self.lofile)
@@ -318,7 +368,7 @@ class LoopbackDisk(Disk):
         if self.device is None:
             return
         logging.debug("Losetup remove %s" % self.device)
-        rc = subprocess.call(["/sbin/losetup", "-d", self.device])
+        rc = subprocess.call([losetupcmd, "-d", self.device])
         self.device = None
 
 
@@ -369,7 +419,7 @@ class Mount:
     def cleanup(self):
         self.unmount()
 
-    def mount(self):
+    def mount(self, options = None):
         pass
 
     def unmount(self):
@@ -386,6 +436,10 @@ class DiskMount(Mount):
 
         self.mounted = False
         self.rmdir   = False
+        if fstype:
+            self.mkfscmd = find_binary_path("mkfs." + self.fstype)
+        else:
+            self.mkfscmd = None
 
     def cleanup(self):
         Mount.cleanup(self)
@@ -394,8 +448,8 @@ class DiskMount(Mount):
     def unmount(self):
         if self.mounted:
             logging.debug("Unmounting directory %s" % self.mountdir)
-            subprocess.call(["/bin/sync"]) # sync the data on this mount point
-            rc = subprocess.call(["/bin/umount", "-l", self.mountdir])
+            subprocess.call([synccmd]) # sync the data on this mount point
+            rc = subprocess.call([umountcmd, "-l", self.mountdir])
             if rc == 0:
                 self.mounted = False
             else:
@@ -412,7 +466,7 @@ class DiskMount(Mount):
         self.disk.create()
 
 
-    def mount(self):
+    def mount(self, options = None):
         if self.mounted:
             return
 
@@ -424,14 +478,17 @@ class DiskMount(Mount):
         self.__create()
 
         logging.debug("Mounting %s at %s" % (self.disk.device, self.mountdir))
-        args = [ "/bin/mount", self.disk.device, self.mountdir ]
+        if options:
+            args = [ mountcmd, "-o", options, self.disk.device, self.mountdir ]
+        else:
+            args = [ mountcmd, self.disk.device, self.mountdir ]
         if self.fstype:
             args.extend(["-t", self.fstype])
 
         rc = subprocess.call(args)
         if rc != 0:
-            raise MountError("Failed to mount '%s' to '%s'" %
-                             (self.disk.device, self.mountdir))
+            raise MountError("Failed to mount '%s' to '%s' with command '%s'. Retval: %s" %
+                             (self.disk.device, self.mountdir, " ".join(args), rc))
 
         self.mounted = True
 
@@ -443,6 +500,8 @@ class ExtDiskMount(DiskMount):
         self.fslabel = fslabel.replace("/", "")
         self.uuid  = None
         self.skipformat = skipformat
+        self.dumpe2fs = find_binary_path("dumpe2fs")
+        self.tune2fs = find_binary_path("tune2fs")
 
     def __parse_field(self, output, field):
         for line in output.split("\n"):
@@ -456,7 +515,7 @@ class ExtDiskMount(DiskMount):
             logging.debug("Skip filesystem format.")
             return
         logging.debug("Formating %s filesystem on %s" % (self.fstype, self.disk.device))
-        rc = subprocess.call(["/sbin/mkfs." + self.fstype,
+        rc = subprocess.call([self.mkfscmd,
                               "-F", "-L", self.fslabel,
                               "-m", "1", "-b", str(self.blocksize),
                               self.disk.device])
@@ -466,7 +525,7 @@ class ExtDiskMount(DiskMount):
 
         dev_null = os.open("/dev/null", os.O_WRONLY)
         try:
-            out = subprocess.Popen(['/sbin/dumpe2fs', '-h', self.disk.device],
+            out = subprocess.Popen([self.dumpe2fs, '-h', self.disk.device],
                                    stdout = subprocess.PIPE,
                                    stderr = dev_null).communicate()[0]
         finally:
@@ -474,7 +533,7 @@ class ExtDiskMount(DiskMount):
 
         self.uuid = self.__parse_field(out, "Filesystem UUID")
         logging.debug("Tuning filesystem on %s" % self.disk.device)
-        subprocess.call(["/sbin/tune2fs", "-c0", "-i0", "-Odir_index",
+        subprocess.call([self.tune2fs, "-c0", "-i0", "-Odir_index",
                          "-ouser_xattr,acl", self.disk.device])
 
     def __resize_filesystem(self, size = None):
@@ -506,9 +565,9 @@ class ExtDiskMount(DiskMount):
         else:
             self.__format_filesystem()
 
-    def mount(self):
+    def mount(self, options = None):
         self.__create()
-        DiskMount.mount(self)
+        DiskMount.mount(self, options)
 
     def __fsck(self):
         logging.debug("Checking filesystem %s" % self.disk.lofile)
@@ -517,7 +576,7 @@ class ExtDiskMount(DiskMount):
     def __get_size_from_filesystem(self):
         dev_null = os.open("/dev/null", os.O_WRONLY)
         try:
-            out = subprocess.Popen(['/sbin/dumpe2fs', '-h', self.disk.lofile],
+            out = subprocess.Popen([self.dumpe2fs, '-h', self.disk.lofile],
                                    stdout = subprocess.PIPE,
                                    stderr = dev_null).communicate()[0]
         finally:
@@ -558,16 +617,14 @@ class VfatDiskMount(DiskMount):
         self.fslabel = fslabel.replace("/", "")
         self.uuid = "%08X" % int(time.time())
         self.skipformat = skipformat
+        self.fsckcmd = find_binary_path("fsck." + self.fstype)
 
     def __format_filesystem(self):
         if self.skipformat:
             logging.debug("Skip filesystem format.")
             return
         logging.debug("Formating %s filesystem on %s" % (self.fstype, self.disk.device))
-        blah = ["/sbin/mkfs." + self.fstype,
-                              "-n", self.fslabel,
-                              "-i", self.uuid,
-                              self.disk.device]
+        blah = [self.mkfscmd, "-n", self.fslabel, "-i", self.uuid, self.disk.device]
         rc = subprocess.call(blah)
         if rc != 0:
             raise MountError("Error creating %s filesystem" % (self.fstype,))
@@ -602,13 +659,13 @@ class VfatDiskMount(DiskMount):
         else:
             self.__format_filesystem()
 
-    def mount(self):
+    def mount(self, options = None):
         self.__create()
-        DiskMount.mount(self)
+        DiskMount.mount(self, options)
 
     def __fsck(self):
         logging.debug("Checking filesystem %s" % self.disk.lofile)
-        subprocess.call(["/sbin/fsck." + self.fstype, "-y", self.disk.lofile])
+        subprocess.call([self.fsckcmd, "-y", self.disk.lofile])
 
     def __get_size_from_filesystem(self):
         return self.disk.size
@@ -640,6 +697,8 @@ class BtrfsDiskMount(DiskMount):
         self.fslabel = fslabel.replace("/", "")
         self.uuid  = None
         self.skipformat = skipformat
+        self.blkidcmd = find_binary_path("blkid")
+        self.btrfsckcmd = find_binary_path("btrfsck")
 
     def __check_btrfs(self):
         found = False
@@ -650,7 +709,7 @@ class BtrfsDiskMount(DiskMount):
                 found = True
                 break
         if not found:
-           raise MountError("Your system can't mount btrfs filesystem")
+            raise MountError("Your system can't mount btrfs filesystem, please make sure your kernel has btrfs support and the module btrfs.ko has been loaded.")
 
         # disable selinux, selinux will block write
         if os.path.exists("/usr/sbin/setenforce"):
@@ -676,7 +735,7 @@ class BtrfsDiskMount(DiskMount):
 
         dev_null = os.open("/dev/null", os.O_WRONLY)
         try:
-            out = subprocess.Popen(['/sbin/blkid', self.disk.device],
+            out = subprocess.Popen([self.blkidcmd, self.disk.device],
                                    stdout = subprocess.PIPE,
                                    stderr = dev_null).communicate()[0]
         finally:
@@ -711,13 +770,15 @@ class BtrfsDiskMount(DiskMount):
         else:
             self.__format_filesystem()
 
-    def mount(self):
+    def mount(self, options = None):
         self.__create()
-        DiskMount.mount(self)
+        if not options:
+            options = "compress"
+        DiskMount.mount(self, options)
 
     def __fsck(self):
         logging.debug("Checking filesystem %s" % self.disk.lofile)
-        subprocess.call(["/sbin/btrfsck", self.disk.lofile])
+        subprocess.call([self.btrfsckcmd, self.disk.lofile])
 
     def __get_size_from_filesystem(self):
         return self.disk.size
@@ -741,6 +802,7 @@ class DeviceMapperSnapshot(object):
 
         self.__created = False
         self.__name = None
+        self.dmsetupcmd = find_binary_path("dmsetup")
 
         """Load dm_snapshot if it isn't loaded"""
         load_module("dm_snapshot")
@@ -767,7 +829,7 @@ class DeviceMapperSnapshot(object):
                                              self.imgloop.device,
                                              self.cowloop.device)
 
-        args = ["/sbin/dmsetup", "create", self.__name, "--table", table]
+        args = [self.dmsetupcmd, "create", self.__name, "--table", table]
         if subprocess.call(args) != 0:
             self.cowloop.cleanup()
             self.imgloop.cleanup()
@@ -780,7 +842,7 @@ class DeviceMapperSnapshot(object):
         if not self.__created:
             return
 
-        rc = subprocess.call(["/sbin/dmsetup", "remove", self.__name])
+        rc = subprocess.call([self.dmsetupcmd, "remove", self.__name])
         if not ignore_errors and rc != 0:
             raise SnapshotError("Could not remove snapshot device")
 
@@ -796,7 +858,7 @@ class DeviceMapperSnapshot(object):
 
         dev_null = os.open("/dev/null", os.O_WRONLY)
         try:
-            out = subprocess.Popen(["/sbin/dmsetup", "status", self.__name],
+            out = subprocess.Popen([self.dmsetupcmd, "status", self.__name],
                                    stdout = subprocess.PIPE,
                                    stderr = dev_null).communicate()[0]
         finally:
@@ -862,7 +924,7 @@ def load_module(module):
     if not found:
         print "Loading %s..." % module
         dev_null = os.open("/dev/null", os.O_WRONLY)
-        modprobe = subprocess.Popen(["/sbin/modprobe", module],
+        modprobe = subprocess.Popen([modprobecmd, module],
                      stdout=dev_null, stderr=dev_null)
         os.waitpid(modprobe.pid, 0)
         os.close(dev_null)
@@ -873,11 +935,11 @@ def myurlgrab(url, filename, proxies):
         file = url.replace("file://", "")
         if not os.path.exists(file):
             raise CreatorError("URLGrabber error: can't find file %s" % file)
-        subprocess.call(["/bin/cp", "-f", file, filename])
+        subprocess.call([copycmd, "-f", file, filename])
     else:
         try:
             filename = g.urlgrab(url = url, filename = filename,
-                proxies = proxies, http_headers = (('Pragma', 'no-cache'),))
+                proxies = proxies, http_headers = (('Pragma', 'no-cache'),), progress_obj = TextProgress())
         except URLGrabError, e:
             raise CreatorError("URLGrabber error: %s: %s" % (e, url))
         except:
