@@ -384,12 +384,13 @@ class MoblinRepoConfig(KickstartConfig):
             fd.write("baseurl=" + baseurl + "\n")
         if mirrorlist:
             fd.write("mirrorlist=" + mirrorlist + "\n")
-        if repo.proxy:
-            fd.write("proxy=" + repo.proxy + "\n")
-        if repo.proxy_username:
-            fd.write("proxy_username=" + repo.proxy_username + "\n")
-        if repo.proxy_password:
-            fd.write("proxy_password=" + repo.proxy_password + "\n")
+        """ Skip saving proxy settings """
+        #if repo.proxy:
+        #    fd.write("proxy=" + repo.proxy + "\n")
+        #if repo.proxy_username:
+        #    fd.write("proxy_username=" + repo.proxy_username + "\n")
+        #if repo.proxy_password:
+        #    fd.write("proxy_password=" + repo.proxy_password + "\n")
         if repo.gpgkey:
             fd.write("gpgkey=" + repo.gpgkey + "\n")
             fd.write("gpgcheck=1\n")
@@ -412,11 +413,19 @@ class MoblinRepoConfig(KickstartConfig):
             self.__create_repo_section(repo, "source", f)
         f.close()
 
-    def apply(self, ksrepo):
+    def apply(self, ksrepo, repodata):
         for repo in ksrepo.repoList:
             if repo.save:
-                self.__create_repo_file(repo, "/etc/yum.repos.d")
+                #self.__create_repo_file(repo, "/etc/yum.repos.d")
                 self.__create_repo_file(repo, "/etc/zypp/repos.d")
+        """ Import repo gpg keys """
+        if repodata:
+            dev_null = os.open("/dev/null", os.O_WRONLY)
+            for repo in repodata:
+                if repo['repokey']:
+                    subprocess.call([fs.find_binary_path("rpm"), "--root=%s" % self.instroot, "--import", repo['repokey']],
+                                    stdout = dev_null, stderr = dev_null)
+            os.close(dev_null)
 
 class RPMMacroConfig(KickstartConfig):
     """A class to apply the specified rpm macros to the filesystem"""
