@@ -83,13 +83,7 @@ class LiveImageCreatorBase(LoopImageCreator):
         if self.ks:
             self.__modules.extend(kickstart.get_modules(self.ks))
 
-        self._dep_checks.extend([
-                            ('/usr/bin/isohybrid',),
-                            ('/usr/sbin/unsquashfs', '/usr/bin/unsquashfs'),
-                            ('/sbin/mksquashfs', '/usr/bin/mksquashfs'),
-                            ('/bin/dd',),
-                            ('/usr/bin/mkisofs',)
-                           ])
+        self._dep_checks.extend(["isohybrid", "unsquashfs", "mksquashfs", "dd", "mkisofs"])
 
     #
     # Hooks for subclasses
@@ -156,15 +150,7 @@ class LiveImageCreatorBase(LoopImageCreator):
 
     def _uncompress_squashfs(self, squashfsimg, outdir):
         """Uncompress file system from squshfs image"""
-
-        unsquashfs = "/usr/sbin/unsquashfs"
-        if not os.path.exists(unsquashfs):
-            unsquashfs = "/usr/bin/unsquashfs"
-            if not os.path.exists(unsquashfs):
-                unsquashfs = "/sbin/unsquashfs"
-                if not os.path.exists(unsquashfs):
-                    raise CreatorError("You didn't install squashfs-tools.");
-
+        unsquashfs = find_binary_path("unsquashfs")
         args = [unsquashfs, "-d", outdir, squashfsimg ]
         rc = subprocess.call(args)
         if (rc != 0):
@@ -176,7 +162,7 @@ class LiveImageCreatorBase(LoopImageCreator):
         """Support Image Convertor"""
         if self.actasconvertor:
             if os.path.exists(base_on) and not os.path.isfile(base_on):
-                args = [ "/bin/dd", "if=%s" % base_on, "of=%s" % self._image ]
+                args = [ ddcmd, "if=%s" % base_on, "of=%s" % self._image ]
                 print "dd %s -> %s" % (base_on, self._image)
                 rc = subprocess.call(args)
                 if rc != 0:
@@ -290,8 +276,8 @@ class LiveImageCreatorBase(LoopImageCreator):
 
     def __create_iso(self, isodir):
         iso = self._outdir + "/" + self.name + ".iso"
-
-        args = ["/usr/bin/mkisofs",
+        mkisofs = find_binary_path("mkisofs")
+        args = [mkisofs,
                 "-J", "-r",
                 "-hide-rr-moved", "-hide-joliet-trans-tbl",
                 "-V", self.fslabel,
@@ -304,9 +290,16 @@ class LiveImageCreatorBase(LoopImageCreator):
         if subprocess.call(args) != 0:
             raise CreatorError("ISO creation failed!")
 
-        if os.path.exists("/usr/bin/isohybrid"):
-            hybrid = ["/usr/bin/isohybrid", "-partok", iso ]
-            if subprocess.call(hybrid) != 0:
+        """ It should be ok still even if you haven't isohybrid """
+        isohybrid = None
+        try:
+            isohybrid = find_binary_path("isohybrid")
+        except:
+            pass
+
+        if isohybrid:
+            args = [isohybrid, "-partok", iso ]
+            if subprocess.call(args) != 0:
              	raise CreatorError("Hybrid ISO creation failed!")
 
         self.__implant_md5sum(iso)
@@ -919,14 +912,10 @@ class LiveUSBImageCreator(LiveImageCreator):
     def __init__(self, *args):
         LiveImageCreator.__init__(self, *args)
 
-        self._dep_checks.extend([
-                            ('/sbin/kpartx',),
-                            ('/sbin/parted', '/usr/sbin/parted')
-                           ])
+        self._dep_checks.extend(["kpartx", "parted"])
         # remove dependency of mkisofs in parent class
-        self._dep_checks = filter(lambda item: \
-                                    not any(map(lambda s: \
-                                        'mkisofs' in s, item)), self._dep_checks)
+        if "mkisofs" in self._dep_checks:
+            self._dep_checks.remove("mkisofs")
 
     def _create_usbimg(self, isodir):
         overlaysizemb = self.overlaysizemb
@@ -948,11 +937,11 @@ class LiveUSBImageCreator(LiveImageCreator):
         livesize = get_file_size(isodir + "/LiveOS")
         if skipcompress:
             tmpmnt = self._mkdtemp("squashfs-mnt")
-            rc = subprocess.call(["/usr/bin/mount", "-o", "loop", isodir + "/LiveOS/squashfs.img", tmpmnt]);
+            rc = subprocess.call([mountcmd, "-o", "loop", isodir + "/LiveOS/squashfs.img", tmpmnt]);
             if rc:
                 raise CreatorError("Can't mount %s" % (isodir + "/LiveOS/squashfs.img"))
             livesize = get_file_size(tmpmnt + "/LiveOS/ext3fs.img")
-            rc = subprocess.call(["/bin/umount", tmpmnt]);
+            rc = subprocess.call([umountcmd, tmpmnt]);
             if rc:
                 raise CreatorError("Can't umount %s" % (tmpmnt))
         usbimgsize = (overlaysizemb + homesizemb + swapsizemb + livesize + plussize) * 1024L * 1024L
@@ -971,11 +960,11 @@ class LiveUSBImageCreator(LiveImageCreator):
             makedirs(usbmnt + "/LiveOS")
             if skipcompress:
                 if os.path.exists(isodir + "/LiveOS/squashfs.img"):
-                    rc = subprocess.call(["/usr/bin/mount", "-o", "loop", isodir + "/LiveOS/squashfs.img", tmpmnt]);
+                    rc = subprocess.call([mountcmd, "-o", "loop", isodir + "/LiveOS/squashfs.img", tmpmnt]);
                     if rc:
                         raise CreatorError("Can't mount %s" % (isodir + "/LiveOS/squashfs.img"))
                     shutil.copyfile(tmpmnt + "/LiveOS/ext3fs.img", usbmnt + "/LiveOS/ext3fs.img")
-                    rc = subprocess.call(["/bin/umount", tmpmnt]);
+                    rc = subprocess.call([umountcmd, tmpmnt]);
                     if rc:
                         raise CreatorError("Can't umount %s" % (tmpmnt))
                 else:
@@ -999,7 +988,7 @@ class LiveUSBImageCreator(LiveImageCreator):
                 usblabel = "UUID=%s" % diskmount.uuid
                 overlaysuffix = "-%s-%s" % (diskmount.fslabel, diskmount.uuid)
 
-            args = ["/bin/cp", "-Rf", isodir + "/isolinux", usbmnt + "/syslinux"]
+            args = [copycmd, "-Rf", isodir + "/isolinux", usbmnt + "/syslinux"]
             rc = subprocess.call(args)
             if rc:
                 raise CreatorError("Can't copy isolinux directory %s" % (isodir + "/isolinux/*"))
@@ -1015,7 +1004,7 @@ class LiveUSBImageCreator(LiveImageCreator):
             for f in ("isolinux.bin", "vesamenu.c32"):
                 path = os.path.join(syslinux_path, f)
                 if os.path.isfile(path):
-                    args = ["/bin/cp", path, usbmnt + "/syslinux/"]
+                    args = [copycmd, path, usbmnt + "/syslinux/"]
                     rc = subprocess.call(args)
                     if rc:
                         raise CreatorError("Can't copy syslinux file %s" % (path))
@@ -1037,9 +1026,9 @@ class LiveUSBImageCreator(LiveImageCreator):
                 print "Initializing persistent overlay file"
                 overfile = "overlay" + overlaysuffix
                 if fstype == "vfat":
-                    args = ["/bin/dd", "if=/dev/zero", "of=" + usbmnt + "/LiveOS/" + overfile, "count=%d" % overlaysizemb, "bs=1M"]
+                    args = [ddcmd, "if=/dev/zero", "of=" + usbmnt + "/LiveOS/" + overfile, "count=%d" % overlaysizemb, "bs=1M"]
                 else:
-                    args = ["/bin/dd", "if=/dev/null", "of=" + usbmnt + "/LiveOS/" + overfile, "count=1", "bs=1M", "seek=%d" % overlaysizemb]
+                    args = [ddcmd, "if=/dev/null", "of=" + usbmnt + "/LiveOS/" + overfile, "count=1", "bs=1M", "seek=%d" % overlaysizemb]
                 rc = subprocess.call(args)
                 if rc:
                     raise CreatorError("Can't create overlay file")
@@ -1049,7 +1038,7 @@ class LiveUSBImageCreator(LiveImageCreator):
             if swapsizemb > 0:
                 print "Initializing swap file"
                 swapfile = usbmnt + "/LiveOS/" + "swap.img"
-                args = ["/bin/dd", "if=/dev/zero", "of=" + swapfile, "count=%d" % swapsizemb, "bs=1M"]
+                args = [ddcmd, "if=/dev/zero", "of=" + swapfile, "count=%d" % swapsizemb, "bs=1M"]
                 rc = subprocess.call(args)
                 if rc:
                     raise CreatorError("Can't create swap file")
@@ -1062,32 +1051,36 @@ class LiveUSBImageCreator(LiveImageCreator):
                 print "Initializing persistent /home"
                 homefile = usbmnt + "/LiveOS/" + homefile
                 if fstype == "vfat":
-                    args = ["/bin/dd", "if=/dev/zero", "of=" + homefile, "count=%d" % homesizemb, "bs=1M"]
+                    args = [ddcmd, "if=/dev/zero", "of=" + homefile, "count=%d" % homesizemb, "bs=1M"]
                 else:
-                    args = ["/bin/dd", "if=/dev/null", "of=" + homefile, "count=1", "bs=1M", "seek=%d" % homesizemb]
+                    args = [ddcmd, "if=/dev/null", "of=" + homefile, "count=1", "bs=1M", "seek=%d" % homesizemb]
                 rc = subprocess.call(args)
                 if rc:
                     raise CreatorError("Can't create home file")
 
+                mkfscmd = find_binary_path("/sbin/mkfs." + fstype)
                 if fstype == "ext2" or fstype == "ext3":
-                    args = ["/sbin/mkfs." + fstype, "-F", "-j", homefile]
+                    args = [mkfscmd, "-F", "-j", homefile]
                 else:
-                    args = ["/sbin/mkfs." + fstype, homefile]
+                    args = [mkfscmd, homefile]
                 rc = subprocess.call(args)
                 if rc:
                     raise CreatorError("Can't mke2fs home file")
                 if fstype == "ext2" or fstype == "ext3":
-                    args = ["/sbin/tune2fs", "-c0", "-i0", "-ouser_xattr,acl", homefile]
+                    tune2fs = find_binary_path("tune2fs")
+                    args = [tune2fs, "-c0", "-i0", "-ouser_xattr,acl", homefile]
                     rc = subprocess.call(args)
                     if rc:
                          raise CreatorError("Can't tune2fs home file")
 
             if fstype == "vfat" or fstype == "msdos":
+                syslinuxcmd = find_binary_path("syslinux")
                 syslinuxcfg = usbmnt + "/syslinux/syslinux.cfg"
-                args = ["/usr/bin/syslinux", "-d", "syslinux", usbloop.partitions[0]["device"]]
+                args = [syslinuxcmd, "-d", "syslinux", usbloop.partitions[0]["device"]]
             elif fstype == "ext2" or fstype == "ext3":
+                extlinuxcmd = find_binary_path("extlinux")
                 syslinuxcfg = usbmnt + "/syslinux/extlinux.conf"
-                args = ["/sbin/extlinux", "-i", usbmnt + "/syslinux"]
+                args = [extlinuxcmd, "-i", usbmnt + "/syslinux"]
             else:
                 raise CreatorError("Invalid file system type: %s" % (fstype))
 
@@ -1111,7 +1104,7 @@ class LiveUSBImageCreator(LiveImageCreator):
                 d = usbloop.disks[dev]
                 loopdev = d['disk'].device
                 break
-            args = ["/bin/dd", "if=" + mbrfile, "of=" + loopdev, "bs=%d" % (mbrsize), "count=1"]
+            args = [ddcmd, "if=" + mbrfile, "of=" + loopdev, "bs=%d" % (mbrsize), "count=1"]
             rc = subprocess.call(args)
             if rc:
                 raise CreatorError("Can't set MBR.")
@@ -1151,7 +1144,7 @@ class LiveUSBImageCreator(LiveImageCreator):
         """Support Image Convertor"""
         if self.actasconvertor:
             if os.path.exists(base_on) and not os.path.isfile(base_on):
-                args = [ "/bin/dd", "if=%s" % base_on, "of=%s" % self._image ]
+                args = [ ddcmd, "if=%s" % base_on, "of=%s" % self._image ]
                 print "dd %s -> %s" % (base_on, self._image)
                 rc = subprocess.call(args)
                 if rc != 0:
