@@ -68,7 +68,7 @@ def get_image_type(path):
         return maptab["vdi"]
 
     dev_null = os.open("/dev/null", os.O_WRONLY)
-    args = [ "/usr/bin/file", path ]
+    args = [ filecmd, path ]
     file = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=dev_null)
     output = file.communicate()[0]
     os.close(dev_null)
@@ -92,11 +92,7 @@ def get_image_type(path):
 
 def get_file_size(file):
     """Return size in MB unit"""
-    du = "/usr/bin/du"
-    if not os.path.exists(du):
-        du = "/bin/du"
-        if not os.path.exists(du):
-            raise CreatorError("command du isn't available")
+    du = find_binary_path("du")
     dev_null = os.open("/dev/null", os.O_WRONLY)
     duProc = subprocess.Popen([du, "-s", "-b", "-B", "1M", file],
                                stdout=subprocess.PIPE, stderr=dev_null)
@@ -128,16 +124,10 @@ def convert_image(srcimg, srcfmt, dstimg, dstfmt):
         raise CreatorError("Invalid destination image format: %s" % dstfmt)
     logging.debug("converting %s image to %s" % (srcimg, dstimg))
     if srcfmt == "vmdk":
-        path = "/usr/bin/qemu-img"
-        if not os.path.isfile(path):
-            raise CreatorError("qemu not installed : "
-                               "%s not found" % path)
+        path = find_binary_path("qemu-img")
         argv = [path, "convert", "-f", "vmdk", srcimg, "-O", dstfmt,  dstimg]
     elif srcfmt == "vdi":
-        path = "/usr/bin/VBoxManage"
-        if not os.path.isfile(path):
-            raise CreatorError("VirtualBox not installed : "
-                               "%s not found" % path)
+        path = find_binary_path("VBoxManage")
         argv = [path, "internalcommands", "converttoraw", srcimg, dstimg]
     else:
         raise CreatorError("Invalid soure image format: %s" % srcfmt)
@@ -188,7 +178,7 @@ def oldmycopytree(src, dst, symlinks=False):
                 oldmycopytree(srcname, dstname, symlinks)
             elif not os.path.isfile(srcname):
                 if not os.path.exists(dstname):
-                    args = [ "/bin/cp" , "-af", srcname, dstname ]
+                    args = [ copycmd, "-af", srcname, dstname ]
                     subprocess.call(args)
             else:
                 shutil.copy2(srcname, dstname)
@@ -210,16 +200,17 @@ def oldmycopytree(src, dst, symlinks=False):
         raise MyError, errors
 
 def mycopytree(src, dst):
+    tarcmd = find_binary_path("tar")
     dev_null = os.open("/dev/null", os.O_WRONLY)
     tmpdir = mkdtemp()
-    args = [ "/bin/tar", "-jcf", tmpdir + "/fs.tar.bz2", "-C", src ]
+    args = [ tarcmd, "-jcf", tmpdir + "/fs.tar.bz2", "-C", src ]
     excludes = []
     ignores = ["dev/fd", "dev/stdin", "dev/stdout", "dev/stderr", "etc/mtab"]
     for exclude in ignores:
         excludes = excludes + ["--exclude", exclude]
     dirnames = os.listdir(src)
     subprocess.call(args + excludes + dirnames, stdout=dev_null, stderr=dev_null)
-    args = [ "/bin/tar", "-jxf", tmpdir + "/fs.tar.bz2", "-C", dst ]
+    args = [ tarcmd, "-jxf", tmpdir + "/fs.tar.bz2", "-C", dst ]
     subprocess.call(args, stdout=dev_null, stderr=dev_null)
     subprocess.call(args)
     os.close(dev_null)
@@ -229,7 +220,7 @@ def myxcopytree(src, dst):
     dev_null = os.open("/dev/null", os.O_WRONLY)
     dirnames = os.listdir(src)
     for dir in dirnames:
-        args = [ "/bin/cp", "-af", src + "/" + dir, dst ]
+        args = [ copycmd, "-af", src + "/" + dir, dst ]
         subprocess.call(args, stdout=dev_null, stderr=dev_null)
     os.close(dev_null)
     ignores = ["dev/fd", "dev/stdin", "dev/stdout", "dev/stderr", "etc/mtab"]
@@ -239,14 +230,7 @@ def myxcopytree(src, dst):
 
 def uncompress_squashfs(squashfsimg, outdir):
     """Uncompress file system from squshfs image"""
-    unsquashfs = "/usr/sbin/unsquashfs"
-    if not os.path.exists(unsquashfs):
-        unsquashfs = "/usr/bin/unsquashfs"
-        if not os.path.exists(unsquashfs):
-            unsquashfs = "/sbin/unsquashfs"
-            if not os.path.exists(unsquashfs):
-                raise SquashfsError("%s didn't exist." % unsquashfs)
-
+    unsquashfs = find_binary_path("unsquashfs")
     args = [ unsquashfs, "-d", outdir, squashfsimg ]
     rc = subprocess.call(args)
     if (rc != 0):
@@ -342,6 +326,8 @@ def set_noproxy_list():
     if not _my_noproxy:
         return
     for item in _my_noproxy.split(","):
+        if not item:
+            continue
         if item[0] != '.' and item.find("/") == -1:
             """ Need to match it """
             _my_noproxy_list.append({"match":0,"needle":item.strip()})
@@ -354,7 +340,7 @@ def set_noproxy_list():
             ip = ip_to_int(needle)
             netmask = 0
             mask = item[item.find("/")+1:].strip()
-         
+
             if mask.isdigit():
                 netmask = int(mask)
                 netmask = ~((1<<(32-netmask)) - 1)
@@ -387,7 +373,7 @@ def isnoproxy(url):
                 return True
     return False
 
-def set_proxies(proxy = None, no_proxy = None): 
+def set_proxies(proxy = None, no_proxy = None):
     _set_proxies(proxy, no_proxy)
     set_noproxy_list()
 
@@ -429,7 +415,7 @@ def remap_repostr(repostr, siteconf):
                     return repostr
 
     return repostr
-    
+
 
 def get_temp_reponame(baseurl):
     md5obj = hashlib.md5(baseurl)
@@ -488,7 +474,7 @@ def get_repostr(repo, siteconf = None):
 
     if repostr.find("--name=") == -1:
         repostr += " --name=%s" % get_temp_reponame(url)
-            
+
     return repostr
 
 def read_siteconf(siteconf = None):
@@ -597,9 +583,27 @@ def get_repostrs_from_siteconf(siteconf):
                 site_repos.append(repostr[1:])
     return site_repos
 
+def get_uncompressed_data_from_url(url, filename, proxies):
+    filename = myurlgrab(url, filename, proxies)
+    suffix = None
+    if filename.endswith(".gz"):
+        suffix = ".gz"
+        subprocess.call([gunzip, "-f", filename])
+    elif filename.endswith(".bz2"):
+        suffix = ".bz2"
+        subprocess.call([bunzip2, "-f", filename])
+    if suffix:
+        filename = filename.replace(suffix, "")
+    return filename
+
+def get_metadata_from_repo(baseurl, proxies, cachedir, reponame, filename):
+    url = str(baseurl + "/" + filename)
+    filename_tmp = str("%s/%s/%s" % (cachedir, reponame, os.path.basename(filename)))
+    return get_uncompressed_data_from_url(url,filename_tmp,proxies)
+
 def get_metadata_from_repos(repostrs, cachedir):
     if not cachedir:
-        cachedir = os.path.abspath(os.path.expanduser("~/.mic2_cachedir"))
+        CreatorError("No cache dir defined.")
 
     my_repo_metadata = []
     for repostr in repostrs:
@@ -623,7 +627,6 @@ def get_metadata_from_repos(repostrs, cachedir):
         if proxy:
            proxies = {str(proxy.split(":")[0]):str(proxy)}
         makedirs(cachedir + "/" + reponame)
-        print "Geting repomd.xml from repo %s..." % reponame
         url = str(baseurl + "/repodata/repomd.xml")
         filename = str("%s/%s/repomd.xml" % (cachedir, reponame))
         repomd = myurlgrab(url, filename, proxies)
@@ -663,62 +666,27 @@ def get_metadata_from_repos(repostrs, cachedir):
                 if elm.attrib["type"] == "primary":
                     primary_type=".xml"
                     break
-        
+
         if not primary_type:
             continue
 
-        print "Getting primary%s from repo %s..." % (primary_type, reponame)
         primary = elm.find("%slocation" % ns).attrib['href']
-        suffix = None
-        if primary.endswith(".gz"):
-            suffix = ".gz"
-        if primary.endswith(".bz2"):
-            suffix = ".bz2"
-        url = str(baseurl + "/" + primary)
-        filename = str("%s/%s/%s" % (cachedir, reponame, os.path.basename(primary)))
-        primary = myurlgrab(url, filename, proxies)
-        if suffix == ".bz2":
-            subprocess.call([bunzip2, "-f", primary])
-        if suffix == ".gz":
-            subprocess.call([gunzip, "-f", primary])
-        if suffix:
-            primary = primary.replace(suffix, "")
+        primary = get_metadata_from_repo(baseurl, proxies, cachedir, reponame, primary)
 
         if patterns:
-            print "Getting patterns %s from repo %s..." % (patterns, reponame)
-            suffix = None
-            if patterns.endswith(".gz"):
-                suffix = ".gz"
-            if patterns.endswith(".bz2"):
-               suffix = ".bz2"
-            url = str(baseurl + "/" + patterns)
-            filename = str("%s/%s/%s" % (cachedir, reponame, os.path.basename(patterns)))
-            patterns = myurlgrab(url, filename, proxies)
-            if suffix == ".bz2":
-                subprocess.call([bunzip2, "-f", patterns])
-            if suffix == ".gz":
-                subprocess.call([gunzip, "-f", patterns])
-            if suffix:
-                patterns = patterns.replace(suffix, "")
+            patterns = get_metadata_from_repo(baseurl, proxies, cachedir, reponame, patterns)
 
         if comps:
-            print "Getting comps %s from repo %s..." % (comps, reponame)
-            suffix = None
-            if comps.endswith(".gz"):
-                suffix = ".gz"
-            if comps.endswith(".bz2"):
-               suffix = ".bz2"
-            url = str(baseurl + "/" + comps)
-            filename = str("%s/%s/%s" % (cachedir, reponame, os.path.basename(comps)))
-            comps = myurlgrab(url, filename, proxies)
-            if suffix == ".bz2":
-                subprocess.call([bunzip2, "-f", comps])
-            if suffix == ".gz":
-                subprocess.call([gunzip, "-f", comps])
-            if suffix:
-                comps = comps.replace(suffix, "")
+            comps = get_metadata_from_repo(baseurl, proxies, cachedir, reponame, comps)
 
-        my_repo_metadata.append({"name":reponame, "baseurl":baseurl, "repomd":repomd, "primary":primary, "cachedir":cachedir, "proxies":proxies, "patterns":patterns, "comps":comps})
+        """ Get repo key """
+        try:
+            repokey = get_metadata_from_repo(baseurl, proxies, cachedir, reponame, "repodata/repomd.xml.key")
+        except CreatorError:
+            repokey = None
+            print "Warning: can't get %s/%s" % (baseurl, "repodata/repomd.xml.key")
+
+        my_repo_metadata.append({"name":reponame, "baseurl":baseurl, "repomd":repomd, "primary":primary, "cachedir":cachedir, "proxies":proxies, "patterns":patterns, "comps":comps, "repokey":repokey})
     return my_repo_metadata
 
 def get_arch(repometadata):
@@ -741,7 +709,7 @@ def get_arch(repometadata):
 
             con.close()
     return archlist
-        
+
 
 def get_package(pkg, repometadata):
     ver = ""
@@ -778,16 +746,55 @@ def get_package(pkg, repometadata):
           pkg = myurlgrab(url, filename, target_repo["proxies"])
           return pkg
     else:
-	return None
+          return None
+
+def get_source_name(pkg, repometadata):
+
+    def get_bin_name(pkg):
+        m = re.match("(.*)\.(.*)\ (.*)-(.*)", pkg)
+        if m:
+            return m.group(1)
+        return None
+
+    def get_src_name(srpm):
+        m = re.match("(.*)-(\d+.*)-(\d+\.\d+).src.rpm", srpm)
+        if m:
+            return m.group(1)
+        return None
+
+    ver = ""
+    target_repo = None
+
+    pkg_name = get_bin_name(pkg)
+
+    if not pkg_name:
+        return None
+
+    for repo in repometadata:
+        if repo["primary"].endswith(".sqlite"):
+            con = sqlite.connect(repo["primary"])
+            for row in con.execute("select version, release, rpm_sourcerpm from packages where name = \"%s\" and arch != \"src\"" % pkg_name):
+                tmpver = "%s-%s" % (row[0], row[1])
+                if tmpver > ver:
+                    pkgpath = "%s" % row[2]
+                    target_repo = repo
+                break
+            con.close()
+    if target_repo:
+        return get_src_name(pkgpath)
+    else:
+        return None
 
 def get_release_no(repometadata, distro="meego"):
+    cpio = find_binary_path("cpio")
+    rpm2cpio = find_binary_path("rpm2cpio")
     release_pkg = get_package("%s-release" % distro, repometadata)
     if release_pkg:
         tmpdir = mkdtemp()
         oldcwd = os.getcwd()
         os.chdir(tmpdir)
-        p1 = subprocess.Popen(["/usr/bin/rpm2cpio", release_pkg], stdout = subprocess.PIPE)
-        p2 = subprocess.Popen(["/bin/cpio", "-idv"], stdin = p1.stdout, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
+        p1 = subprocess.Popen([rpm2cpio, release_pkg], stdout = subprocess.PIPE)
+        p2 = subprocess.Popen([cpio, "-idv"], stdin = p1.stdout, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
         p2.communicate()
         f = open("%s/etc/%s-release" % (tmpdir, distro), "r")
         content = f.read()
@@ -797,7 +804,7 @@ def get_release_no(repometadata, distro="meego"):
         return content.split(" ")[2]
     else:
         return "UNKNOWN"
-        
+
 def get_kickstarts_from_repos(repometadata):
     kickstarts = []
     for repo in repometadata:
@@ -818,21 +825,10 @@ def get_kickstarts_from_repos(repometadata):
 
         location = elm.find("%slocation" % ns)
         image_config = str(repo["baseurl"] + "/" + location.attrib["href"])
-        suffix = ""
-        if image_config.endswith(".gz"):
-            suffix = ".gz"
-        if image_config.endswith(".bz2"):
-            suffix = ".bz2"
-        
         filename = str("%s/%s/image-config.xml%s" % (repo["cachedir"], repo["name"], suffix))
-        image_config = myurlgrab(image_config, filename, repo["proxies"])
-        if suffix == ".bz2":
-            subprocess.call([bunzip2, "-f", image_config])
-        if suffix == ".gz":
-            subprocess.call([gunzip, "-f", image_config])
-        if suffix:
-            image_config = image_config.replace(suffix, "")
-        
+
+        image_config = get_uncompressed_data_from_url(image_config,filename,repo["proxies"])
+
         try:
             root = xmlparse(image_config)
         except SyntaxError:
@@ -905,54 +901,93 @@ def get_pkglist_in_group(group, patterns):
 
     return pkglist
 
+def is_statically_linked(binary):
+    ret = False
+    dev_null = os.open("/dev/null", os.O_WRONLY);
+    args = [ filecmd, binary ]
+    file = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=dev_null)
+    output = file.communicate()[0]
+    os.close(dev_null)
+    if output.find(", statically linked, ") > 0:
+        ret = True
+    return ret
+
 def setup_qemu_emulator(rootdir, arch):
     # mount binfmt_misc if it doesn't exist
     if not os.path.exists("/proc/sys/fs/binfmt_misc"):
-        subprocess.call(["/sbin/modprobe", "binfmt_misc"])
+        subprocess.call([modprobecmd, "binfmt_misc"])
     if not os.path.exists("/proc/sys/fs/binfmt_misc/register"):
-        subprocess.call(["/bin/mount", "-t", "binfmt_misc", "none", "/proc/sys/fs/binfmt_misc"])
-    # qemu emulator should be a static-linked executable file
-    qemu_emulator = "/usr/bin/qemu-arm-static"
+        subprocess.call([mountcmd, "-t", "binfmt_misc", "none", "/proc/sys/fs/binfmt_misc"])
+
+    # qemu_emulator is a special case, we can't use find_binary_path
+    # qemu emulator should be a statically-linked executable file
+    qemu_emulator = "/usr/bin/qemu-arm"
+    if not os.path.exists(qemu_emulator) or not is_statically_linked(qemu_emulator):
+        qemu_emulator = "/usr/bin/qemu-arm-static"
     if not os.path.exists(qemu_emulator):
-        raise CreatorError("Please install qemu-arm-static")
+        raise CreatorError("Please install a statically-linked qemu-arm")
     if not os.path.exists(rootdir + "/usr/bin"):
         makedirs(rootdir + "/usr/bin")
     shutil.copy(qemu_emulator, rootdir + qemu_emulator)
 
-    # unregister it if it has been registered
-    if os.path.exists("/proc/sys/fs/binfmt_misc/qemu-arm-static"):
+    # disable selinux, selinux will block qemu emulator to run
+    if os.path.exists("/usr/sbin/setenforce"):
+        subprocess.call(["/usr/sbin/setenforce", "0"])
+
+    node = "/proc/sys/fs/binfmt_misc/arm"
+    if is_statically_linked(qemu_emulator) and os.path.exists(node):
+        return qemu_emulator
+
+    # unregister it if it has been registered and is a dynamically-linked executable
+    if not is_statically_linked(qemu_emulator) and os.path.exists(node):
         qemu_unregister_string = "-1\n"
-        fd = open("/proc/sys/fs/binfmt_misc/qemu-arm-static", "w")
+        fd = open("/proc/sys/fs/binfmt_misc/arm", "w")
         fd.write(qemu_unregister_string)
         fd.close()
 
     # register qemu emulator for interpreting other arch executable file
-    qemu_arm_string = ":qemu-arm-static:M::\\x7fELF\\x01\\x01\\x01\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x02\\x00\\x28\\x00:\\xff\\xff\\xff\\xff\\xff\\xff\\xff\\x00\\xff\\xff\\xff\\xff\\xff\\xff\\xff\\xff\\xfa\\xff\\xff\\xff:/usr/bin/qemu-arm-static:\n"
-    fd = open("/proc/sys/fs/binfmt_misc/register", "w")
-    fd.write(qemu_arm_string)
-    fd.close()
+    if not os.path.exists(node):
+        qemu_arm_string = ":arm:M::\\x7fELF\\x01\\x01\\x01\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x02\\x00\\x28\\x00:\\xff\\xff\\xff\\xff\\xff\\xff\\xff\\x00\\xff\\xff\\xff\\xff\\xff\\xff\\xff\\xff\\xfa\\xff\\xff\\xff:%s:\n" % qemu_emulator
+        fd = open("/proc/sys/fs/binfmt_misc/register", "w")
+        fd.write(qemu_arm_string)
+        fd.close()
 
-    # disable selinux, selinux will block qemu-arm-static to run
-    if os.path.exists("/usr/sbin/setenforce"):
-        subprocess.call(["/usr/sbin/setenforce", "0"])
+    return qemu_emulator
+
+def create_release(config, destdir, name, outimages):
+    """ TODO: This functionality should really be in creator.py inside the
+    ImageCreator class. """
+
+    # For virtual machine images, we have a subdir for it, this is unnecessary
+    # for release
+    thatsubdir = None
+    for i in range(len(outimages)):
+        file = outimages[i]
+        if not os.path.isdir(file) and os.path.dirname(file) != destdir:
+            thatsubdir = os.path.dirname(file)
+            newfile = os.path.join(destdir, os.path.basename(file))
+            shutil.move(file, newfile)
+            outimages[i] = newfile
+    if thatsubdir:
+        shutil.rmtree(thatsubdir, ignore_errors = True)
 
-def create_release(config, destdir, name):
     """ Create release directory and files """
     os.system ("cp %s %s/%s.ks" % (config, destdir, name))
+    outimages.append("%s/%s.ks" % (destdir,name))
+
     os.system ("mv %s/*-pkgs.txt %s/%s.packages" % (destdir, destdir, name))
+    outimages.append("%s/%s.packages" % (destdir,name))
+
     d = os.listdir(destdir)
     for f in d:
-        if os.path.isdir(destdir+"/"+f):
-            shutil.rmtree("%s/%s" %(destdir, f ))
-            continue
-        if ".iso" in f:
+        if f.endswith(".iso"):
             ff = f.replace(".iso", ".img")
             os.rename("%s/%s" %(destdir, f ), "%s/%s" %(destdir, ff))
-        elif ".usbimg" in f:
+            outimages.append("%s/%s" %(destdir, ff))
+        elif f.endswith(".usbimg"):
             ff = f.replace(".usbimg", ".img")
             os.rename("%s/%s" %(destdir, f ), "%s/%s" %(destdir, ff))
-        elif "-sda.bin" in f or "-boot.bin" in f:
-            os.remove(destdir+"/"+f)
+            outimages.append("%s/%s" %(destdir, ff))
 
     fd = open(destdir + "/MANIFEST", "w")
     d = os.listdir(destdir)
@@ -969,4 +1004,33 @@ def create_release(config, destdir, name):
                 md5sum = md5sum.split(" ")[0]
                 fd.write(md5sum+" "+f+"\n")
 
+    outimages.append("%s/MANIFEST" % destdir)
     fd.close()
+
+    """ Update the file list. """
+    updated_list = []
+    for file in outimages:
+        if os.path.exists("%s" % file):
+            updated_list.append(file)
+
+    return updated_list
+
+def get_local_distro():
+    print "Local linux distribution:"
+    for file in glob.glob("/etc/*-release"):
+        fd = open(file, "r")
+        content = fd.read()
+        fd.close()
+        print content
+    if os.path.exists("/etc/issue"):
+        fd = open("/etc/issue", "r")
+        content = fd.read()
+        fd.close()
+        print content
+    print "Local Kernel version: " + os.uname()[2]
+
+def check_mic_installation(argv):
+    creator_name = os.path.basename(argv[0])
+    if os.path.exists("/usr/local/bin/" + creator_name) \
+        and os.path.exists("/usr/bin/" + creator_name):
+        raise CreatorError("There are two mic2 installations existing, this will result in some unpredictable errors, the reason is installation path of mic2 binary is different from  installation path of mic2 source on debian-based distros, please remove one of them to ensure it can work normally.")
